@@ -11,9 +11,11 @@ BEGIN {
 
 BEGIN {
 	$Types::Common::String::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Common::String::VERSION   = '1.002002';
+	$Types::Common::String::VERSION   = '1.010002';
 }
 
+$Types::Common::String::VERSION =~ tr/_//d;
+
 use Type::Library -base, -declare => qw(
 	SimpleStr
 	NonEmptySimpleStr
@@ -25,6 +27,7 @@ use Type::Library -base, -declare => qw(
 	NonEmptyStr
 	LowerCaseStr
 	UpperCaseStr
+	StrLength
 );
 
 use Type::Tiny ();
@@ -91,7 +94,7 @@ $meta->add_type(
 	inlined    => sub {
 		if ($nestr) {
 			my $xsub = Type::Tiny::XS::get_subname_for($_[0]->name);
-			return "$xsub($_[1])" if $xsub;
+			return "$xsub($_[1])" if $xsub && !$Type::Tiny::AvoidCallbacks;
 		}
 		undef, qq(length($_) > 0);
 	},
@@ -128,7 +131,7 @@ $meta->add_type(
 	parent     => NonEmptySimpleStr,
 	constraint => sub { !/\p{Upper}/ms },
 	inlined    => sub { undef, qq($_ !~ /\\p{Upper}/ms) },
-	message    => sub { "Must not contain pper case letters" },
+	message    => sub { "Must not contain upper case letters" },
 );
 
 LowerCaseSimpleStr->coercion->add_type_coercions(
@@ -147,6 +150,65 @@ UpperCaseSimpleStr->coercion->add_type_coercions(
 	NonEmptySimpleStr, q[ uc($_) ],
 );
 
+$meta->add_type(
+	name       => StrLength,
+	parent     => Str,
+	constraint_generator => sub {
+		return $meta->get_type('StrLength') unless @_;
+		
+		my ($min, $max) = @_;
+		Types::Standard::Int->check($_)
+			|| Types::Standard::_croak("Parameters for StrLength[`min, `max] expected to be integers; got $_")
+			for @_;
+		
+		if (defined $max) {
+			return sub { length($_[0]) >= $min and length($_[0]) <= $max };
+		}
+		else {
+			return sub { length($_[0]) >= $min };
+		}
+	},
+	inline_generator => sub {
+		my ($min, $max) = @_;
+		
+		return sub {
+			my $v = $_[1];
+			my @code = (undef); # parent constraint
+			push @code, "length($v) >= $min";
+			push @code, "length($v) <= $max" if defined $max;
+			return @code;
+		};
+	},
+	deep_explanation => sub {
+		my ($type, $value, $varname) = @_;
+		my ($min, $max) = @{ $type->parameters || [] };
+		my @whines;
+		if (defined $max) {
+			push @whines, sprintf(
+				'"%s" expects length(%s) to be between %d and %d',
+				$type,
+				$varname,
+				$min,
+				$max,
+			);
+		}
+		else {
+			push @whines, sprintf(
+				'"%s" expects length(%s) to be at least %d',
+				$type,
+				$varname,
+				$min,
+			);
+		}
+		push @whines, sprintf(
+			"length(%s) is %d",
+			$varname,
+			length($value),
+		);
+		return \@whines;
+	},
+);
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -177,25 +239,72 @@ L<MooseX::Types::Common::String>.
 
 =over
 
-=item C<SimpleStr>
+=item *
 
-=item C<NonEmptySimpleStr>
+B<SimpleStr>
 
-=item C<NumericCode>
+=item *
 
-=item C<LowerCaseSimpleStr>
+B<NonEmptySimpleStr>
 
-=item C<UpperCaseSimpleStr>
+=item *
 
-=item C<Password>
+B<NumericCode>
 
-=item C<StrongPassword>
+=item *
 
-=item C<NonEmptyStr>
+B<LowerCaseSimpleStr>
 
-=item C<LowerCaseStr>
+=item *
 
-=item C<UpperCaseStr>
+B<UpperCaseSimpleStr>
+
+=item *
+
+B<Password>
+
+=item *
+
+B<StrongPassword>
+
+=item *
+
+B<NonEmptyStr>
+
+=item *
+
+B<LowerCaseStr>
+
+=item *
+
+B<UpperCaseStr>
+
+=back
+
+This module also defines an extra type constraint not found in
+L<MooseX::Types::Common::String>.
+
+=over
+
+=item *
+
+B<< StrLength[`min, `max] >>
+
+Type constraint for a string between min and max characters long. For
+example:
+
+  StrLength[4, 20]
+
+It is sometimes useful to combine this with another type constraint in an
+intersection.
+
+  (LowerCaseStr) & (StrLength[4, 20])
+
+The max length can be omitted.
+
+  StrLength[10]   # at least 10 characters
+
+Lengths are inclusive.
 
 =back
 
@@ -218,7 +327,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
